# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"RXNFingerprintContainer": "00_core.ipynb",
         "FingerprintGenerator": "00_core.ipynb",
         "get_default_tokenizer": "01_tokenization.ipynb",
         "SmilesTokenizer": "01_tokenization.ipynb",
         "BasicSmilesTokenizer": "01_tokenization.ipynb",
         "load_vocab": "01_tokenization.ipynb",
         "convert_reaction_to_valid_features": "01_tokenization.ipynb",
         "convert_reaction_to_valid_features_batch": "01_tokenization.ipynb",
         "SMI_REGEX_PATTERN": "01_tokenization.ipynb",
         "InputFeatures": "01_tokenization.ipynb",
         "InputFeaturesBatch": "01_tokenization.ipynb",
         "NotCanonicalizableSmilesException": "01_tokenization.ipynb",
         "canonicalize_smi": "01_tokenization.ipynb",
         "process_reaction": "01_tokenization.ipynb",
         "RXNBERTFingerprintGenerator": "02_transformer_fingerprints.ipynb",
         "RXNBERTMinhashFingerprintGenerator": "02_transformer_fingerprints.ipynb",
         "get_default_model_and_tokenizer": "02_transformer_fingerprints.ipynb",
         "generate_fingerprints": "02_transformer_fingerprints.ipynb"}

modules = ["core.py",
           "tokenization.py",
           "transformer_fingerprints.py"]

doc_url = "https://pschwllr # change that to rxn4chemistry.github.io/rxnfp/"

git_url = "https://github.com/pschwllr # change that to rxn4chemistry/rxnfp/tree/master/"

def custom_doc_links(name): return None
